/*
    Cryptography Export macros

    Copyright (c) 2007 by Charles Connell <charles@connells.org>

    Kopete    (c) 2002-2007 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU Lesser General Public            *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/
#ifndef CRYPTOGRAPHYEXPORT_H
#define CRYPTOGRAPHYEXPORT_H

#include <kdemacros.h>

#define KOPETECRYPTOGRAPHY_EXPORT KDE_EXPORT

#endif
