@import url(./Light-Noback.css);

body	{
	font-size: 12px; font-family: "DejaVu Sans Mono", monospace; font-style: normal; font-weight: normal; margin: 0;
	background: url("../images/kopete.png"); background-position: center; background-attachment: fixed; background-repeat: no-repeat; background-color: white
	}