/*
    kopeteidleplatform_p.h  -  Kopete Idle Platform

    Copyright (c) 2008      by Roman Jarosz          <kedgedev@centrum.cz>
    Kopete    (c) 2008      by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU Lesser General Public            *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/
#ifndef KOPETEIDLEPLATFORM_P_H
#define KOPETEIDLEPLATFORM_P_H

namespace Kopete {
class IdlePlatform
{
public:
    IdlePlatform();
    ~IdlePlatform();

    bool init();
    int secondsIdle();

private:
    class Private;
    Private *const d;
};
}

#endif
