body 
{
	font-family:Verdana, sans-serif;
	font-size : 12px;
}

.contenu_incoming .corps_message
{
	text-decoration: none; 
	padding-top: 2 px;
	margin: 5px 15px 5px 15px;
}

.contenu_outgoing .corps_message
{
	text-decoration: none; 
	padding-top: 2 px;
	margin: 5px 15px 5px 15px;
}

.status
{
	margin-left: 15px;
	font: 11px Verdana, sans-serif;
}

.avatar 
{
    	position: relative;
    	top: 0;
    	left: 0;
	height: 80px;
}

.nom 
{
	position:relative;
	top: -35px;
	margin-bottom: -30px;
	z-index:2;
}
    
.texte
{
	position:relative;
	top: -35px;
	margin-left: 100px;
	margin-bottom: -10px;
	z-index:1;
}