/*
    tests.h

    Copyright (c) 2007      by Charles Connell        <charles@connells.org>

    Kopete    (c) 2007      by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef _PIPES_TESTS_H_
#define _PIPES_TESTS_H_

#include <QString>

class KopeteXmlParserTests
{
public:
    /*
    * Change body to a fixed string
    * Change body foreground color to red
    */
    static QString test1(const QString &str);
};

#endif
