@import url(../main.css);

body	{
	font-size: 12px; font-family: "DejaVu Sans Mono", monospace; font-style: normal; font-weight: normal; margin: 0;
	background-color: black
	}
.stateSendingImg { width: 16px; height: 16px; background: transparent url("../images/state-sending-white.gif") no-repeat center center; }
