//
// C++ Implementation: task_take_test
//
// Description:
//
//
// Author: Kopete Developers <kopete-devel@kde.org>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

//#include "requesttask.h"

int main()
{
    // balls, root task requires client, will test in situ instead
}
